unit forms.index;

{$mode ObjFPC}

interface

uses
  SysUtils, Classes, Rtl.HTMLActions, htmlfragment, Web, fprpcclient, processservice, fpprocesscapturepoller;

type

  { TfrmIndex }

  TfrmIndex = class(THTMLFragment)
    actbtnCancel: THTMLElementAction;
    actbtnStart: THTMLElementAction;
    actedtA: THTMLElementAction;
    actedtB: THTMLElementAction;
    alForm: THTMLElementActionList;
    rpc: TPas2jsRPCClient;
    procedure actbtnCancelExecute(Sender: TObject; Event: TJSEvent);
    procedure actbtnStartExecute(Sender: TObject; Event: TJSEvent);
    procedure DataModuleCreate(Sender: TObject);
  private
    Service : TprocesscontrolService;
    FPoller : TProcessCapturePoller;
    FJobID : String;
    procedure DoDoutput(Sender: TObject; aOutput: String);
    procedure DoGetStatus(Sender: TObject; aProcessID: String; aOffset: NativeInt);
    procedure DoProcessDone(Sender: TObject; aStatus: TProcessStatus; aExitCode: Integer);
    procedure DoStatusFail(Sender: TObject; aError: String);
  public

  end;

var
  frmIndex: TfrmIndex;

implementation

uses js;

{$R *.lfm}

{ TfrmIndex }

procedure TfrmIndex.DoGetStatus(Sender: TObject; aProcessID: String; aOffset: NativeInt);


  procedure DoStatusFail(Sender: TObject; const aError: TRPCError);
  begin
    FPoller.ReportProgressFail(aError.Message);
  end;

  procedure DoStatusOK(aResult: JSValue);

  const statuses : array[Boolean] of TProcessStatus
                 =  (psError,psRunning);

  Var
    D : TJSObject absolute aResult;
    aExitCode : Integer;
    aNewOffset : NativeInt;
    aOutput : string;
    aStatus : TProcessStatus;

  begin
    aOutput:=String(D['output']);
    aExitCode:=NativeInt(D['status']);
    aNewOffset:=NativeInt(D['offset']);
    aStatus:=Statuses[aExitCode=-1];
    FPoller.ReportProgress(aStatus,aOutput,aExitCode,aNewOffset)
  end;

begin
  service.GetStatus(FJobID,aOffset,@DoStatusOK,@DoStatusFail);
end;

procedure TfrmIndex.DoProcessDone(Sender: TObject; aStatus: TProcessStatus; aExitCode: Integer);

Const
  Exits : Array[TProcessStatus] of string
        = ('Running','Exited','Error');

begin
  Write('Process ',Exits[aStatus]);
  if aStatus=psExited then
    Writeln(' with exit code ',aExitCode)
  else
    Writeln();
end;

procedure TfrmIndex.DoStatusFail(Sender: TObject; aError: String);
begin
  Writeln('Error getting status: ',aError);
end;

procedure TfrmIndex.DoDoutput(Sender: TObject; aOutput: String);
begin
  Writeln(aOutput);
end;

procedure TfrmIndex.actbtnStartExecute(Sender: TObject; Event: TJSEvent);


  procedure DoStartFail(Sender: TObject; const aError: TRPCError);
  begin
    Writeln('Failed to start process : ',aError.Message);
  end;

  procedure DoStartOK(aResult: JSValue);

  begin
    FJobID:=String(aResult);
    FPoller.ProcessID:=FJobID;
    FPoller.Start;
  end;

var
  a,b : string;

begin
  a:=actedtA.Value;
  b:=actedtB.Value;
  Service.StartProcess(A,B,@DoStartOK,@DoStartFail);
end;

procedure TfrmIndex.actbtnCancelExecute(Sender: TObject; Event: TJSEvent);
begin
  Writeln('Canceled wait for process.');
  FPoller.Cancel;
end;

procedure TfrmIndex.DataModuleCreate(Sender: TObject);
begin
  Service:=TprocesscontrolService.Create(Self);
  Service.RPCClient:=RPC;
  FPoller:=TProcessCapturePoller.Create(Self);
  FPoller.OnProcessOutput:=@DoDoutput;
  FPoller.OnGetProcessStatus:=@DoGetStatus;
  FPoller.OnProcessDone:=@DoProcessDone;
  FPoller.OnStatusFail:=@DoStatusFail;
end;

end.

