unit remoteexecutor;

{$mode ObjFPC}

interface

uses
  Js, Classes, SysUtils, fprpcclient, processservice, fpprocesscapturepoller;

Type
  TProcessStatus = fpprocesscapturepoller.TProcessStatus;

  { TRemoteExecutor }

  TRemoteExecutor = class(TCustomProcessCapturePoller)
  private
    FService: TprocesscontrolService;
    function GetClient: TRPCClient;
    procedure SetClient(AValue: TRPCClient);
  Protected
    procedure DoStatusCheck; override;
    Property Service : TprocesscontrolService Read FService;
  Public
    Constructor Create(aOwner : TComponent); override;
    Procedure Execute(a,b : String);
    Destructor Destroy; override;
  Published
    Property RPCClient : TRPCClient Read GetClient Write SetClient;
    Property OnProcessDone;
    Property OnProcessOutput;
    Property OnStatusFail;
    Property LinebasedOutput;
    Property PollInterval;
    Property MaxFailCount;
    Property MaxCheckCount;
  end;

Const
  psError   = TProcessStatus.psError;
  psExited  = TProcessStatus.psExited;
  prRunning = TProcessStatus.psRunning;

implementation

{ TRemoteExecutor }

function TRemoteExecutor.GetClient: TRPCClient;
begin
  Result:=FService.RPCClient;
end;

procedure TRemoteExecutor.SetClient(AValue: TRPCClient);
begin
  FService.RPCClient:=aValue;
end;

procedure TRemoteExecutor.DoStatusCheck;

  procedure DoStatusFail(Sender: TObject; const aError: TRPCError);
  begin
    ReportProgressFail(aError.Message);
  end;

  procedure DoStatusOK(aResult: JSValue);

  Var
    D : TJSObject absolute aResult;
    aExitCode : Integer;
    aNewOffset : NativeInt;
    aOutput : string;
    aStatus : TProcessStatus;

  begin
    aOutput:=String(D['output']);
    aExitCode:=NativeInt(D['status']);
    aNewOffset:=NativeInt(D['offset']);
    case aExitCode of
     -1 : aStatus:=psRunning;
     0 : aStatus:=psExited;
    else
      aStatus:=psError;
    end;
    DoReportProgress(aStatus,aOutput,aExitCode,aNewOffset)
  end;

begin
  service.GetStatus(ProcessID,OutputOffset,@DoStatusOK,@DoStatusFail);
end;

constructor TRemoteExecutor.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FService:=TprocesscontrolService.Create(Self);
end;

procedure TRemoteExecutor.Execute(a, b: String);

  procedure DoStartFail(Sender: TObject; const aError: TRPCError);
  begin
    SetFailCount(MaxFailCount);
    ReportProgressFail(aError.Message);
  end;

  procedure DoStartOK(aResult: JSValue);

  begin
    ProcessID:=String(aResult);
    Start;
  end;

begin
  Service.StartProcess(A,B,@DoStartOK,@DoStartFail);
end;

destructor TRemoteExecutor.Destroy;
begin
  FreeAndNil(FService);
  inherited Destroy;
end;

end.

