unit fpprocesscapture; 

{$mode objfpc}{$H+}

interface

uses
{$IFDEF UNIX}
  baseunix,unix,
{$ENDIF}  
  SysUtils, Classes, Process;

Const
  NullFile = '/dev/null';
  SCaptureProcess = 'proc-output';
  SStatusText = 'Status : ';
  SSignalText = 'Signal : ';
  StatusExt = '.status';
  LogExt = '.log';
  PidExt = '.pid';

Type

  { TProcessCapture }

  TProcessCapture = Class(TComponent)
  private
    FInputFile: String;
    FLogDir: String;
    FWorkingDir: String;
    FOutputCodePage : TSystemCodePage;
    function SafeReadLine(const AFileName: String): String;
  Protected
    Function CreateArgs(Exe : String; Args : TStrings) : PPchar;
    Function GetLogDir : String;
    Function GetInputFile : String;
    Function SafeWrite(Const AFileName,ALine : String) : Boolean;
  Public
    Constructor Create(aOwner : TComponent); override;
    Function Execute(Exe : String; Args: Array of string) : string;
    Function Execute(Exe : String; Args: TStrings) : string;
    Function CleanupProcess(Const AProcess : String) : Boolean;
    Function GetOutputFile(Const AProcess : String) : String;
    Function GetPidFile(Const AProcess : String) : String;
    Function GetStatusFile(Const AProcess : String) : String;
    Function GetProcessID(Const AProcess : String) : Integer;
    Function IsProcessRunning(Const AProcess : String) : Boolean;
    Function GetProcessExitStatus(Const AProcess : String) : Integer;
    Function GetProcessOutput(Const AProcess : String; Var AOffSet : Integer) : RawByteString;
  Published
    Property LogDir : String Read FLogDir Write FLogDir;
    Property InputFile : String Read FInputFile Write FInputFile;
    Property WorkingDir : String Read FWorkingDir Write FWorkingDir;
    Property OutputCodePage : TSystemCodePage Read FOutputCodePage Write FOutputCodePage;
  end;


implementation


function TProcessCapture.CreateArgs(Exe: String; Args: TStrings): PPchar;

Var
  P : PPChar;
  I : Integer;

begin
  Result:=GetMem((Args.Count+3)*SizeOf(PChar));
  P:=Result;
  P^:=Pchar(Exe);
  For I:=0 to Args.Count-1 do
    begin
    Inc(P);
    P^:=Pchar(Args[i]);
    end;
  Inc(P);
  P^:=Nil;
end;

function TProcessCapture.GetLogDir: String;
begin
  Result:=FLogDir;
  If (Result='') then
    begin
    Result:=IncludeTrailingPathDelimiter(GetTempDir(True))+SCaptureProcess;
    If not ForceDirectories(Result) then
      Result:=GetTempDir(True);
    end;
  if (Result<>'') then
    Result:=IncludeTrailingPathDelimiter(Result);
end;

function TProcessCapture.GetInputFile: String;
begin
  Result:=FInputFile;
  If (Result='') then
    Result:=NullFile;
end;

function TProcessCapture.SafeWrite(const AFileName, ALine: String): Boolean;

Var
  F : Text;

begin
  Result:=False;
  AssignFile(F,AFileName);
  {$i-}
  Rewrite(F);
  if (IOResult=0) then
     begin
     Writeln(F,ALine);
     if (IOResult=0) then
       begin
       close(f);
       if (IOResult=0) then
         Result:=True;
       end;
     end;
  {$i+}
end;

constructor TProcessCapture.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FOutputCodePage:=DefaultSystemCodePage;
end;

function TProcessCapture.Execute(Exe: String; Args: array of string): string;

Var
  S : TStrings;
  I : Integer;

begin
  S:=TStringList.Create;
  try
    For I:=Low(Args) to High(Args) do
      S.Add(Args[i]);
    Result:=Execute(Exe,S);
  finally
    S.Free;
  end;
end;

{$IFNDEF USEFORK}
{
  Param 1 : Exe
  Param 2 : WorkingDir
  Param 3 : OutputFile
  Param 4 : Inputfile
  Param 5 : StatusFile
  Param 6 : PIDFile
  Param 7 - : Args to be passed to program

}

function TProcessCapture.Execute(Exe : String; Args: TStrings) : string;

Var
  G : TGUID;
  i : integer;
  Helper,D,T : String;
  P : TProcess;

begin

  Helper:=ExtractFilePath(ParamStr(0))+'taskhelper';
  CreateGUID(G);
  Result:=GUIDToString(G);
  Result:=Copy(Result,2,Length(Result)-2);
  T:=GetLogDir;
  T:=T+Result;
  D:=WorkingDir;
  if D='' then
    D:=GetCurrentDir;
  P:=TProcess.Create(Self);
  try
    P.Executable:=Helper;
    P.Parameters.Add(Exe);
    P.Parameters.Add(D);
    P.Parameters.Add(GetOutputFile(Result));
    P.Parameters.Add(GetInputFile);
    P.Parameters.Add(GetStatusFile(Result));
    P.Parameters.Add(GetPIDFile(Result));
    For I:=0 to Args.Count-1 do
      P.Parameters.Add(Args[i]);
    P.Options:=[poNoConsole];
    P.Execute;
  finally
    P.Free;
  end;
end;

{$ELSE}

function TProcessCapture.Execute(Exe : String; Args: TStrings) : string;

Var
  G : TGUID;
  pid : TPid;
  fin,fout,status : cInt;
  PP : PPchar;
  T : String;

begin
  status:=-1;
  CreateGUID(G);
  Result:=GUIDToString(G);
  Result:=Copy(Result,2,Length(Result)-2);
  T:=GetLogDir;
  T:=T+Result;
  pid:=fpFork;
  if (pid=-1) then
    begin
    Result:='';
    exit
    end
  else if (Pid=0) then
    begin
    fin:=fpOpen(GetInputFile,O_RDONLY);
    if (fin<0) then
      Halt(1);
    fout:=fpOpen(GetOutputFile(Result),O_WRONLY or O_CREAT);
    if (fout<0) then
      Halt(2);
    if (fpdup2(fin,0)<0) then
      Halt(3);
    if (fpdup2(fout,1)<0) then
      Halt(4);
    if (fpdup2(fout,2)<0) then
      Halt(5);
    if (FWorkingDir<>'') then
      if not SetCurrentDir(FWorkingDir) then
        Halt(6);
    pid:=fpfork;
    if (Pid=0) then
      begin
      PP:=CreateArgs(Exe,Args);
      fpExecV(Exe,PP);
      FreeMem(PP);
      Halt(127);
      end
    else
      begin
      SafeWrite(GetPIDFile(Result),IntToStr(Pid));
      pid:=FpWaitPid(pid,status,0);
      if (pid>=0) then
        begin
        if wifexited(Status) then
          T:=SStatusText+IntToStr(wexitStatus(status))
        else if wifsignaled(Status) then
          T:=SSignalText+IntToStr(wtermsig(status));
        SafeWrite(GetStatusFile(Result),T);
        end;
      Halt(0);
      end
    end;
end;
{$ENDIF}

function TProcessCapture.CleanupProcess(const AProcess: String): Boolean;

  Function CleanFile(FN : String) : Boolean;

  begin
    Result:=Not FileExists(FN);
    If not Result then
      Result:=DeleteFile(FN);
  end;

begin
  Result:=CleanFile(GetOutputFile(AProcess));
  Result:=CleanFile(GetPidFile(AProcess)) and Result;
  Result:=CleanFile(GetStatusFile(AProcess)) and Result;
end;

function TProcessCapture.GetOutputFile(const AProcess: String): String;
begin
  Result:=GetLogDir+AProcess+LogExt;
end;

function TProcessCapture.GetPidFile(const AProcess: String): String;
begin
  Result:=GetLogDir+AProcess+PidExt;
end;

function TProcessCapture.GetStatusFile(const AProcess: String): String;
begin
  Result:=GetLogDir+AProcess+StatusExt;
end;

function TProcessCapture.SafeReadLine(const AFileName: String): String;

Var
  F : Text;

begin
  Result:='';
  AssignFile(F,AFileName);
  {$i-}
  Reset(F);
  if (IOResult=0) and Not EOF(F) then
     begin
     ReadLn(F,Result);
     if (IOResult<>0) then
       begin
       Result:='';
       Close(F);
       if (IOResult<>0) then
         Result:='';
       end;
     end;
  {$i+}
end;
function TProcessCapture.GetProcessID(const AProcess: String): Integer;

Var
  FN,S : String;

begin
  Result:=-1;
  FN:=GetPIDFile(AProcess);
  S:=SafeReadLine(FN);
  Result:=StrToIntDef(S,-1);
end;

function TProcessCapture.IsProcessRunning(const AProcess: String): Boolean;
begin
  Result:=Not FileExists(GetStatusFile(AProcess));
end;

function TProcessCapture.GetProcessExitStatus(const AProcess: String): Integer;

Var
  FN,S : String;

begin
  Result:=-1;
  FN:=GetStatusFile(AProcess);
  S:=SafeReadLine(FN);
  if Pos(SStatusText,S)=1 then
    begin
    Delete(S,1,Length(SStatusText));
    Result:=StrTointDef(S,-1);
    end;
end;

function TProcessCapture.GetProcessOutput(const AProcess: String;
  var AOffSet: Integer): RawByteString;

Var
  F : TFileStream;
  Count : Integer;

begin
  Result:='';
  try
    F:=TFileStream.Create(GetOutputFile(AProcess),fmOpenRead);
    try
      Count:=F.Seek(0,soFromEnd);
      F.Seek(AOffset,soFromBeginning);
      SetLength(Result,Count-AOffset);
      if (Count>AOffset) then
        F.ReadBuffer(Result[1],Count-AOffset);
      SetCodePage(Result,OutputCodePage,False);
      AOffset:=Count;
    finally
      F.Free;
    end;
  except
    Result:='';
  end;
end;

end.

