{$mode objfpc}
{$h+}
{$ifdef unix}
uses baseunix, sysutils;
{$else}
{$ifdef windows}
uses sysutils, windows;
{$else}
{$error Taskhelper is not supported on this platform.}
{$endif}
{$endif}

{

  Param 1 : Exe
  Param 2 : WorkingDir
  Param 3 : OutputFile
  Param 4 : Inputfile 
  Param 5 : StatusFile
  Param 6 : PIDFile
  Param 7 - : Args to be passed to program
  
}


const
  SStatusText = 'Status : ';
{$IFDEF UNIX}
  SSignalText = 'Signal : ';
{$ENDIF}

Type
{$IFDEF USEUNICODESTRINGS}
  TCaptureString = UnicodeString;
{$ELSE}
  TCaptureString = AnsiString;
{$ENDIF}

  TCaptureData = Record
    Exe : TCaptureString;
    WorkingDir : TCaptureString;
    OutputFile : TCaptureString;
    InputFile : TCaptureString;
    StatusFile: TCaptureString;
    PIDFile : TCaptureString;
    Args: Array of TCaptureString;
  end;

function SafeWrite(const AFileName: TCaptureString; ALine: String): Boolean;

Var
  F : Text;

begin
  Result:=False;
  AssignFile(F,AFileName);
  {$i-}
  Rewrite(F);
  if (IOResult=0) then
     begin
     Writeln(F,ALine);
     if (IOResult=0) then
       begin
       close(f);
       if (IOResult=0) then
         Result:=True;
       end;
     end;
  {$i+}
end;    

{$IFDEF UNIX}
function CreateArgs(Exe: String; Args: Array of AnsiString): PPchar;

Var
  P : PPChar;
  I : Integer;

begin
  Result:=GetMem((Length(Args)+3)*SizeOf(PChar));
  P:=Result;
  P^:=Pchar(Exe);
  For I:=0 to Length(Args)-1 do
    begin
    Inc(P);
    P^:=Pchar(Args[i]);
    end;
  Inc(P);
  P^:=Nil;
end;

function DoCapture(Data : TCaptureData) : Integer;

var
  pid : TPid;
  fin,fout,status : cInt;
  PP : PPchar;
  T : string;

begin
  pid:=fpFork;
  if (pid=-1) then
    Exit(127)
  else if (Pid=0) then
    begin
    // Child, start new process with in/output redirected.
    fin:=fpOpen(Data.InputFile,O_RDONLY);
    if (fin<0) then
      Halt(1);
    fout:=fpOpen(Data.OutputFile,O_WRONLY or O_CREAT);
    if (fout<0) then
      Halt(2);
    if (fpdup2(fin,0)<0) then
      Halt(3);
    if (fpdup2(fout,1)<0) then
      Halt(4);
    if (fpdup2(fout,2)<0) then
      Halt(5);
    if (Data.WorkingDir<>'') then
      if not SetCurrentDir(Data.WorkingDir) then
        Halt(6);
    pid:=fpGetPID;
    PP:=CreateArgs(Data.Exe,Data.Args);
    fpExecV(Data.Exe,PP);
    FreeMem(PP);
    // Here we need to exit with error code.
    Halt(127);
    end
  else
    begin
    SafeWrite(Data.PIDFile,IntToStr(Pid));
    status:=0;
    pid:=FpWaitPid(pid,status,0);
    if (pid>=0) then
      begin
      if wifexited(Status) then
        T:=SStatusText+IntToStr(wexitStatus(status))
      else if wifsignaled(Status) then
        T:=SSignalText+IntToStr(wtermsig(status));
      SafeWrite(Data.StatusFile,T);
      end;
    Halt(0);
    end;
end;
{$else}
Function MaybeQuoteIfNotQuoted(Const S : TCaptureString) : TCaptureString;

begin
  If (Pos(' ',S)<>0) and (pos('"',S)=0) then
    Result:='"'+S+'"'
  else
    Result:=S;
end;

function createcommandline(Data : TCaptureData) : TCaptureString;

var
  I : integer;

begin
  Result:=MaybeQuoteIfNotQuoted(Data.Exe);
  For I:=0 to Length(Data.Args)-1 do
    Result:=Result+' '+MaybeQuoteIfNotQuoted(Data.Args[i]);
end;

{$IFDEF USEUNICODESTRINGS}
Procedure InitStartupInfo(out SI : STARTUPINFOW; aInput,aOutput : THandle);
{$ELSE}
Procedure InitStartupInfo(out SI : STARTUPINFOA; aInput,aOutput : THandle);
{$ENDIF}

begin
  {$IFDEF USEUNICODESTRINGS}
  SI:=Default(STARTUPINFOW);
  {$ELSE}
  SI:=Default(STARTUPINFOA);
  {$ENDIF}
  SI.cb:=SizeOf(SI);
  SI.dwFlags:=Startf_UseStdHandles;
  SI.hStdInput:=aInput;
  SI.hStdOutput:=aOutput;
  SI.hStdError:=aOutput;
end;

function DoCapture (Data : TCaptureData) : integer;

var
  fin,fout : THandle;
  T : TCaptureString;
  CreationFlags : Cardinal;
  {$IFDEF USEUNICODESTRINGS}
  StartUpInfo : TSTARTUPINFOW;
  {$ELSE}
  StartUpInfo : TSTARTUPINFOA;
  {$ENDIF}
  ProcessInformation : TProcessInformation;
  R,Status : DWord;
  ProcessHandle: THandle;
  ProcessID: Integer;
  sa: TSECURITYATTRIBUTES;


begin
  sa.nLength := sizeof(sa);
  sa.lpSecurityDescriptor := Nil;
  sa.bInheritHandle:=True;
  fin:=CreateFile(PAnsiChar(Data.InputFile),GENERIC_READ,FILE_SHARE_READ,@sa,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
  if (fin=NilHandle) then
    Exit(124);
  fout:=CreateFile(PAnsiChar(Data.OutputFile),GENERIC_WRITE OR GENERIC_READ,FILE_SHARE_WRITE or FILE_SHARE_READ,@sa,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
  if (fout=NilHandle) then
    Exit(123);
  T:=CreateCommandLine(Data);
  CreationFlags:=CREATE_NO_WINDOW;
  InitStartupInfo(StartUpInfo,Fin,FOut);
{$IFDEF USEUNICODESTRINGS}
  if not CreateProcessW(pwidechar(Data.Exe),pwidechar(T),Nil,Nil,True,CreationFlags,Nil,pwidechar(Data.WorkingDir),@StartUpInfo,@ProcessInformation) then
    Exit(127);
{$ELSE}
  if not CreateProcessA(pansichar(Data.Exe),PAnsichar(T),Nil,Nil,True,CreationFlags,Nil,PAnsiChar(Data.WorkingDir),@StartUpInfo,@ProcessInformation) then
    Exit(127);
{$ENDIF}
  ProcessHandle:=ProcessInformation.hProcess;
  ProcessID:=ProcessInformation.dwProcessID;
  SafeWrite(Data.PIDFile,IntToStr(ProcessID));
  R:=WaitForSingleObject (ProcessHandle,Infinite);
  If R=Wait_Failed then
    Exit(126);
  Status:=0;
  if not GetExitCodeProcess(ProcessHandle,Status) then
    Exit(125)
  else
    begin
    T:=SStatusText+IntToStr(status);
    SafeWrite(Data.StatusFile,T);
    end;
  Result:=0;
end;
{$endif}

var
  i : Integer;
  CD : TCaptureData;

begin
  if paramcount<6 then
    Halt(8);
  CD.Exe:=ParamStr(1);
  CD.WorkingDir:=ParamStr(2);
  CD.OutputFile:=ParamStr(3);
  CD.InputFile:=ParamStr(4);
  CD.StatusFile:=ParamStr(5);
  CD.PIDFile:=ParamStr(6);
  CD.Args:=[];
  if ParamCount>6 then
    begin
    SetLength(CD.Args,ParamCount-6);
    For I:=7 to ParamCount do
      CD.Args[I-7]:=Paramstr(I);
    end;  
  Halt(DoCapture(CD));
end.
